package com.example.demo.comments.service;

import java.util.List;
import java.util.Objects;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.videos.model.VideoEntity;
import com.example.demo.videos.service.VideoService;
import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.repository.CommentRepository;

@Service
public class CommentService {
    private final CommentRepository repository;
    private final UserService userService;
    private final VideoService videoService;

    public CommentService(CommentRepository repos, UserService user, VideoService video) {
        repository = repos;
        userService = user;
        videoService = video;
    }

    public List<CommentEntity> getAll(Long videoId) {
        if (Objects.equals(videoId, 0L)) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
        return StreamSupport.stream(repository.findAllByVideoId(videoId).spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<CommentEntity> getAllByUser(Long userId, int page, int size) {
        if (!Objects.equals(userId, 0L)) {
            return repository.findAllByUserId(userId, PageRequest.of(page, size, Sort.by("id")));
        }
        return repository.findAll(PageRequest.of(page, size, Sort.by("id")));
    }

    @Transactional(readOnly = true)
    public Page<CommentEntity> getAllByVideo(Long videoId, int page, int size) {
        if (!Objects.equals(videoId, 0L)) {
            return repository.findAllByVideoId(videoId, PageRequest.of(page, size, Sort.by("id")));
        }
        return repository.findAll(PageRequest.of(page, size, Sort.by("id")));
    }

    public CommentEntity get(Long id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(CommentEntity.class, id));
    }

    public CommentEntity create(CommentEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    public CommentEntity update(Long id, CommentEntity entity) {
        final CommentEntity existsEntity = get(id);
        final VideoEntity video = videoService.get(entity.getVideo().getId());
        final UserEntity user = userService.get(entity.getUser().getId());
        existsEntity.setText(entity.getText());
        existsEntity.setUser(user);
        existsEntity.setVideo(video);
        return repository.save(existsEntity);
    }

    public CommentEntity delete(Long id) {
        final CommentEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }

}
