package com.example.demo.likes.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.likes.model.LikeEntity;
import com.example.demo.likes.service.LikeService;
import com.example.demo.videos.service.VideoService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@RestController
@RequestMapping("/like")
public class LikeController {
    private final LikeService likeService;
    private final UserService userService;
    private final VideoService videoService;
    private final ModelMapper modelMapper;

    public LikeController(LikeService like, UserService user, VideoService video, ModelMapper model) {
        likeService = like;
        userService = user;
        videoService = video;
        modelMapper = model;
    }

    private LikeDto toDto(LikeEntity entity) {
        return modelMapper.map(entity, LikeDto.class);
    }

    private LikeEntity toEntity(LikeDto dto) {
        final LikeEntity entity = modelMapper.map(dto, LikeEntity.class);
        entity.setUser(userService.get(dto.getUserId()));
        entity.setVideo(videoService.get(dto.getVideoId()));
        return entity;
    }

    @GetMapping
    public List<LikeDto> getAll() {
        return likeService.getAll(0L).stream().map(this::toDto).toList();
    }

    @GetMapping("/user/{userId}")
    public List<LikeDto> getAllByUser(@PathVariable(name = "userId") Long userId) {
        return likeService.getAllByUser(userId).stream().map(this::toDto).toList();
    }

    @GetMapping("/video/{videoId}")
    public List<LikeDto> getAllByVideo(@PathVariable(name = "videoId") Long videoId) {
        return likeService.getAll(videoId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public LikeDto get(@PathVariable(name = "id") Long id) {
        return toDto(likeService.get(id));
    }

    @PostMapping
    public LikeDto create(@RequestBody @Valid LikeDto dto) {
        return toDto(likeService.create(toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public LikeDto delete(@PathVariable(name = "id") Long id) {
        return toDto(likeService.delete(id));
    }

    @DeleteMapping("/{userId}/{videoId}")
    public void deleteVideoUser(@PathVariable(name = "userId") Long userId,
            @PathVariable(name = "videoId") Long videoId) {
        likeService.deleteByUserIdAndVideoId(userId, videoId);
    }
}
