package com.example.demo.likes.repository;

import java.util.List;
import java.util.Optional;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import org.springframework.data.repository.PagingAndSortingRepository;
import org.springframework.data.repository.query.Param;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.likes.model.LikeEntity;

public interface LikeRepository extends CrudRepository<LikeEntity, Long>,
        PagingAndSortingRepository<LikeEntity, Long> {
    List<LikeEntity> findAllByUserId(Long userId);

    Page<LikeEntity> findAllByUserId(Long userId, Pageable pageable);

    @Transactional
    @Modifying
    @Query("DELETE FROM LikeEntity l WHERE l.user.id = :userId AND l.video.id = :videoId")
    void deleteByUserIdAndVideoId(@Param("userId") Long userId, @Param("videoId") Long videoId);

    List<LikeEntity> findAllByVideoId(Long videoId);

    Page<LikeEntity> findAllByVideoId(Long videoId, Pageable pageable);

    Optional<LikeEntity> findByUserIdAndVideoId(Long userId, Long videoId);
}
