package com.example.demo.likes.service;

import java.util.List;
import java.util.Objects;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.likes.model.LikeEntity;
import com.example.demo.likes.repository.LikeRepository;

@Service
public class LikeService {
    private final LikeRepository repository;

    public LikeService(LikeRepository repos) {
        repository = repos;
    }

    public List<LikeEntity> getAll(Long videoId) {
        if (Objects.equals(videoId, 0L)) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
        return StreamSupport.stream(repository.findAllByVideoId(videoId).spliterator(), false).toList();
    }

    public List<LikeEntity> getAllByUser(Long userId) {
        return StreamSupport.stream(repository.findAllByUserId(userId).spliterator(), false).toList();
    }

    public Page<LikeEntity> getAll(Long videoId, int page, int size) {
        if (Objects.equals(videoId, 0L)) {
            return repository.findAll(PageRequest.of(page, size, Sort.by("id")));
        }
        return repository.findAllByVideoId(videoId, PageRequest.of(page, size, Sort.by("id")));
    }

    public Page<LikeEntity> getAllByUser(Long userId, int page, int size) {
        return repository.findAllByUserId(userId, PageRequest.of(page, size, Sort.by("id")));
    }

    public LikeEntity get(Long id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(LikeEntity.class, id));
    }

    public LikeEntity create(LikeEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    public LikeEntity delete(Long id) {
        final LikeEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }

    public void deleteByUserIdAndVideoId(Long userId, Long videoId) {
        repository.deleteByUserIdAndVideoId(userId, videoId);
    }

    public boolean findByUserIdAndVideoId(Long userId, Long videoId) {
        return repository.findByUserIdAndVideoId(userId, videoId).isPresent();
    }
}
