package com.example.demo.videos.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import com.example.demo.comments.api.CommentDto;
import com.example.demo.comments.api.CommentUserVideoTitleDto;
import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.service.CommentService;
import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.likes.service.LikeService;
import com.example.demo.users.service.UserService;
import com.example.demo.videos.model.VideoEntity;
import com.example.demo.videos.service.VideoService;

@Controller
@RequestMapping(VideoListController.URL)
public class VideoListController {
    public static final String URL = "/list";
    private static final String VIDEO_VIEW = "video";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String COMMENT_VIEW = "comment";

    private final VideoService videoService;
    private final UserService userService;
    private final ModelMapper modelMapper;
    private final CommentService commentService;
    private final LikeService likeService;

    public VideoListController(VideoService video, ModelMapper model, CommentService comment, LikeService like,
            UserService user) {
        videoService = video;
        modelMapper = model;
        commentService = comment;
        likeService = like;
        userService = user;
    }

    private VideoDtoLike toDto(VideoEntity entity) {
        final Long userId = userService.getUserIdLong();
        final VideoDtoLike dto = new VideoDtoLike();
        dto.setDescription(entity.getDescription());
        dto.setImage(entity.getImage());
        dto.setTitle(entity.getTitle());
        dto.setUserId(entity.getUser().getId());
        dto.setVid(entity.getVid());
        dto.setId(entity.getId());
        boolean flag = likeService.findByUserIdAndVideoId(userId, entity.getId());
        dto.setIsLiked(flag);
        return dto;
    }

    private CommentUserVideoTitleDto toDto(CommentEntity entity) {
        CommentUserVideoTitleDto dto = new CommentUserVideoTitleDto();
        dto.setId(entity.getId());
        dto.setText(entity.getText());
        dto.setUserId(entity.getUser().getId());
        dto.setVideoId(entity.getVideo().getId());
        dto.setUser(entity.getUser().getLogin());
        dto.setTitle(entity.getVideo().getTitle());
        return dto;
    }

    @GetMapping("/video")
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Map<String, Object> attrib = PageAttributesMapper.toAttributes(
                videoService.getAll(0L, page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attrib);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return VIDEO_VIEW;
    }

    @GetMapping("/comment/{videoId}")
    public String getAllComments(@PathVariable(name = "videoId") Long videoId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Map<String, Object> attrib = PageAttributesMapper
                .toAttributes(commentService.getAllByVideo(videoId, page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attrib);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return COMMENT_VIEW;
    }
}
