package com.example.demo.videos.service;

import java.util.List;
import java.util.Objects;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.videos.model.VideoEntity;
import com.example.demo.videos.repository.VideoRepository;

@Service
public class VideoService {
    private final VideoRepository repository;

    public VideoService(VideoRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<VideoEntity> getAll(Long userId) {
        if (Objects.equals(userId, 0L)) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
        return StreamSupport.stream(repository.findByUserId(userId).spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<VideoEntity> getAll(Long userId, int page, int size) {
        if (!Objects.equals(userId, 0L)) {
            return repository.findByUserId(userId, PageRequest.of(page, size, Sort.by("id")));
        }
        return repository.findAll(PageRequest.of(page, size, Sort.by("id")));
    }

    @Transactional(readOnly = true)
    public VideoEntity get(Long id) {
        return repository.findById(id).orElseThrow(() -> new NotFoundException(VideoEntity.class, id));
    }

    @Transactional
    public VideoEntity create(VideoEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public VideoEntity update(Long id, VideoEntity entity) {
        final VideoEntity existsEntity = get(id);
        existsEntity.setUser(entity.getUser());
        existsEntity.setImage(entity.getImage());
        existsEntity.setDescription(entity.getDescription());
        existsEntity.setTitle(entity.getTitle());
        existsEntity.setVid(entity.getVid());
        return repository.save(existsEntity);
    }

    @Transactional
    public VideoEntity delete(Long id) {
        final VideoEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }

}
