package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.service.CommentService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.videos.model.VideoEntity;
import com.example.demo.videos.service.VideoService;

@SpringBootTest
class CommentServiceTests {
    @Autowired
    private VideoService videoService;
    @Autowired
    private UserService userService;
    @Autowired
    private CommentService commentService;

    private CommentEntity comment;
    private VideoEntity video;
    private UserEntity user;

    @BeforeEach
    void createData() {

        user = userService.create(new UserEntity("null", "null"));
        video = videoService.create(new VideoEntity(user, "null", "null", "null", "null"));
        comment = commentService.create(new CommentEntity(video, user, "test"));
        commentService.create(new CommentEntity(video, user, "second"));
        commentService.create(new CommentEntity(video, user, "third"));

    }

    @AfterEach
    void removeData() {
        commentService.getAll(0L).forEach(item -> commentService.delete(item.getId()));
        videoService.getAll(0L).forEach(item -> videoService.delete(item.getId()));
        userService.getAll().forEach(item -> userService.delete(item.getId()));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, commentService.getAll(0L).size());
        Assertions.assertEquals(comment, commentService.get(comment.getId()));
        Assertions.assertEquals(3, commentService.getAll(video.getId()).size());
    }

    @Test
    void createNullableTest() {
        final CommentEntity nullCom = new CommentEntity(video, user, null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> commentService.create(nullCom));
    }

    @Test
    void updateTest() {
        final String test = "TEST";
        final String lastText = comment.getText();
        final CommentEntity updatedEntity = commentService.update(comment.getId(),
                new CommentEntity(video, user, test));

        Assertions.assertEquals(3, commentService.getAll(0L).size());
        Assertions.assertEquals(updatedEntity, commentService.get(comment.getId()));
        Assertions.assertNotEquals(lastText, updatedEntity.getText());
    }
}
