package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.likes.model.LikeEntity;
import com.example.demo.likes.service.LikeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.videos.model.VideoEntity;
import com.example.demo.videos.service.VideoService;

@SpringBootTest
class LikeServiceTests {
    @Autowired
    private VideoService videoService;
    @Autowired
    private UserService userService;
    @Autowired
    private LikeService likeService;

    private LikeEntity like;
    private VideoEntity video;
    private UserEntity user;
    private VideoEntity video1;
    private UserEntity user1;

    @BeforeEach
    void createData() {

        user = userService.create(new UserEntity("null", "null"));
        video = videoService.create(new VideoEntity(user, "null", "null", "null", "null"));
        user1 = userService.create(new UserEntity("aboba", "null"));
        video1 = videoService.create(new VideoEntity(user, "null", "null", "null", "null"));
        like = likeService.create(new LikeEntity(video, user));
        likeService.create(new LikeEntity(video1, user));
        likeService.create(new LikeEntity(video, user1));
        likeService.create(new LikeEntity(video1, user1));
    }

    @AfterEach
    void removeData() {
        likeService.getAll(0L).forEach(item -> likeService.delete(item.getId()));
        videoService.getAll(0L).forEach(item -> videoService.delete(item.getId()));
        userService.getAll().forEach(item -> userService.delete(item.getId()));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(4, likeService.getAll(0L).size());
        Assertions.assertEquals(like, likeService.get(like.getId()));
        Assertions.assertEquals(4, likeService.getAll(0L).size());
    }

    @Test
    void removeOne() {
        likeService.deleteByUserIdAndVideoId(user.getId(), video.getId());
        Assertions.assertEquals(3, likeService.getAll(0L).size());
    }

    @Test
    void getByUser() {
        Assertions.assertEquals(2, likeService.getAll(video.getId()).size());
        Assertions.assertEquals(2, likeService.getAll(video1.getId()).size());
    }

    @Test
    void getByVideo() {
        Assertions.assertEquals(2, likeService.getAllByUser(user.getId()).size());
        Assertions.assertEquals(2, likeService.getAllByUser(user1.getId()).size());
    }
}
