package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.videos.model.VideoEntity;
import com.example.demo.videos.service.VideoService;

@SpringBootTest
class VideoServiceTests {
    @Autowired
    private VideoService videoService;
    @Autowired
    private UserService userService;

    private VideoEntity video;
    private UserEntity user1;
    private UserEntity user2;

    @BeforeEach
    void createData() {
        removeData();

        user1 = userService.create(new UserEntity("mail1", "pass"));
        user2 = userService.create(new UserEntity("mail2", "pass"));
        videoService.create(new VideoEntity(user1, "first", "null", "pas", "pas"));
        videoService.create(new VideoEntity(user2, "second", "null", "pas", "pas"));
        video = videoService.create(new VideoEntity(user2, "third", "null", "pas", "pas"));
    }

    @AfterEach
    void removeData() {
        videoService.getAll(0L).forEach(item -> videoService.delete(item.getId()));
        userService.getAll().forEach(item -> userService.delete(item.getId()));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, videoService.getAll(0L).size());
        Assertions.assertEquals(2, videoService.getAll(user2.getId()).size());
        Assertions.assertEquals(video, videoService.get(video.getId()));
    }

    @Test
    void createNullableTest() {
        final VideoEntity nullableVideo = new VideoEntity(user1, null, null, null, null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> videoService.create(nullableVideo));
    }

    @Test
    void updateTest() {
        final String test = "TEST";
        final String lastTitle = video.getTitle();
        final String lastDesc = video.getDescription();
        final String lastVid = video.getVid();
        final String lastImage = video.getImage();
        final VideoEntity updatedEntity = videoService.update(video.getId(),
                new VideoEntity(user1, test, test, test, test));
        Assertions.assertEquals(3, videoService.getAll(0L).size());
        Assertions.assertEquals(updatedEntity, videoService.get(video.getId()));
        Assertions.assertNotEquals(lastTitle, updatedEntity.getTitle());
        Assertions.assertNotEquals(lastDesc, updatedEntity.getDescription());
        Assertions.assertNotEquals(lastVid, updatedEntity.getVid());
        Assertions.assertNotEquals(lastImage, updatedEntity.getImage());
    }

}
